#####################################################
## NAME: Poly_MBR_Analyzer
## Source Name: Poly_MBR_Analyzer.py
## Version: ArcGIS 9.2
## Author: Environmental Systems Research Institute Inc.
## Usage: Poly_MBR_Analyzer <input_AreaFeatures>, <output_PolygonData>
## Required Arguments: Input polygon layer, output polygon feature classe
##
## Description:  Adds fields that describe a polygon shape's relationship to
## its minimum bounding rectangle. The contents of these fields supports queries
## for determining how to label these features and for how to select these
## features for generalization.
##
## Date: Jan 9, 2007
#####################################################

###Import required modules
import sys, os, locale, string, tempfile, arcgisscripting

if __name__ == "__main__":  
    try:
        #Check the number of parameters.
        if len(sys.argv) < 2:
            raise Exception, "Incorrect number of Parameters"
        
        #Create the Geoprocessing Object
        GP = arcgisscripting.create()

        #Get the Parameters
        HP_5K = GP.GetParameterAsText(0)
        ##HP_5K = 'D:\AdaCnt\92HydroData\test.mdb\Hydro_5K\HydroAreas_5K'
        HP_24K = GP.GetParameterAsText(1)
        ##HP_24K = 'D:\AdaCnt\92HydroData\test.mdb\Hydro_5K\HydroAreas_5Kmbr'

        #Check to see if the data are of the proper dimension
        dsc1 = GP.Describe(HP_5K)
        t1 = dsc1.ShapeType
        
        if t1 != "Polygon":
            ErrorDesc = "Input Hydro Area Features must be a Polygon Feature Class."
            raise
        GP.AddMessage ("  ...Successfully checked Geometry Type.")
        
        #Make sure the Output datasets do not already exist.    
        if GP.Exists(HP_24K):
            GP.Delete_management (HP_24K)
            GP.AddMessage("   ...Deleted Pre-existing Hydro Areas with shape types.")

        ##---------------------
        ## Now we start by copying the polygon data, adding fields to the copy, and calculating them
        ##---------------------
        
        GP.CopyFeatures_management (HP_5K,HP_24K)

        #Add Fields
        tHP5K = GP.Describe(HP_24K)
        #Check to see if the new fields already exist, if not create them
        if not GP.ListFields(HP_24K,"HullWidth").Next():
            GP.AddField_management(HP_24K,"HullWidth","DOUBLE",32,8)
        if not GP.ListFields(HP_24K,"HullLen").Next():
            GP.AddField_management(HP_24K,"HullLen","DOUBLE",32,8)
        if not GP.ListFields(HP_24K,"MBR_Area").Next():
            GP.AddField_management(HP_24K,"MBR_Area","DOUBLE",32,8)
        if not GP.ListFields(HP_24K,"RatioL2W").Next():
            GP.AddField_management(HP_24K,"RatioL2W","DOUBLE",32,8)
        if not GP.ListFields(HP_24K,"PartCount").Next():
            GP.AddField_management(HP_24K,"PartCount","LONG",32,8)

        GP.AddMessage("   ...Checked Shape Characteristics Fields, and now calculating their values")
        ErrorDesc = "Failed while calculating field values"
        #Get Values for Fields
        rows = GP.UpdateCursor(HP_24K)
        row = rows.Next()
        while row:
            pGeom = row.GetValue(tHP5K.ShapeFieldName)
            pGA = pGeom.Area
            pPC = pGeom.PartCount
        
            MbrStr = pGeom.HullRectangle
            MbrCoords = MbrStr.split(' ')
            x0 = float(MbrCoords[0])
            y0 = float(MbrCoords[1])
            x1 = float(MbrCoords[2])
            y1 = float(MbrCoords[3])
            x2 = float(MbrCoords[4])
            y2 = float(MbrCoords[5])
            Dx0 = x0 - x1
            Dy0 = y0 - y1
            Dx1 = x1 - x2
            Dy1 = y1 - y2
            w2 = ((Dx0 * Dx0) + (Dy0 * Dy0)) ** 0.5
            l2 = ((Dx1 * Dx1) + (Dy1 * Dy1)) ** 0.5
            a2 = w2 * l2
            HA = (pGA / a2) * 100
            RatioL2W = l2 / w2
            row.HullWidth = w2
            row.HullLen = l2
            row.MBR_Area = HA
            row.PartCount = pPC
            row.RatioL2W = RatioL2W
            rows.UpdateRow(row)
            row = rows.Next()
        rows = ' '
        HP_24K = ' ' 
        GP.AddMessage("   ...Successfully added shape type metrics to polygon data")

    except Exception, ErrorDesc:
        GP.AddError(str(ErrorDesc))    