#####################################################
## NAME: Minimum Bounding Rectangles for Polygons
## Source Name: MBRRects.py
## Version: ArcGIS 9.2
## Author: Environmental Systems Research Institute Inc.
## Usage: MBRRects <input polygon feature class>,  <output_PolygonData>} 
## Required Arguments: Input layers/features, output polygon & polyline feature classes
##                     
## Description: 
##
##
## Date: Mar 20, 2006
## Updated: Mar, 20, 2006
#####################################################

###Import required modules
###Import required modules
import sys, os, locale, string, tempfile, arcgisscripting

if __name__ == "__main__":  
    try:
        GP = arcgisscripting.create()

        #Get the Parameters
        InputPolys = GP.GetParameterAsText(0)
        ##InputPolys = 'D:\usa\detail1\dtl_st.shp'
        
        OutputPolys = GP.GetParameterAsText(1)
        ##OutputPolys = 'D:\usa\detail1\statembr.shp'
        
        #Check to see if the data are of the proper dimension
        dsc1 = GP.Describe(InputPolys)
        t1 = dsc1.ShapeType
        
        if t1 != "Polygon":
            ErrorDesc = "Input data must be a Polygon Features."
            raise
        GP.AddMessage ("  ...Successfully checked Geometry Type.")
        
        #Make sure the Output dataset does not already exist.    
        if GP.Exists(OutputPolys):
            GP.Delete_management (OutputPolys)
            GP.AddMessage("   ...Deleted pre-existing output.")


        ##---------------------
        ## Now we start by copying the polygon data, adding fields to the copy, and calculating them
        ##---------------------
    
        GP.CopyFeatures_management(InputPolys,OutputPolys)
        
        #Add Fields
        tHP5K = GP.Describe(OutputPolys)
        ##Check to see if the new fields already exist, if not create them

        if not GP.ListFields(OutputPolys,"L2W").Next():
            GP.AddField_management(OutputPolys,"L2W","LONG",32,8)
        if not GP.ListFields(OutputPolys,"MBR").Next():
            GP.AddField_management(OutputPolys,"MBR","LONG",32,8)
        GP.AddMessage("   ...Checked Shape Characteristics Fields, and now calculating their values")
        ErrorDesc = "Failed while calculating field values"
       
        #Process data
        rows = GP.UpdateCursor(OutputPolys)
        row = rows.Next()
        while row:
            pGeom = row.GetValue(tHP5K.ShapeFieldName)
            pGA = pGeom.Area
            pPC = pGeom.PartCount
        
            MbrStr = pGeom.HullRectangle
            MbrCoords = MbrStr.split(' ')
            x0 = float(MbrCoords[0])
            y0 = float(MbrCoords[1])
            x1 = float(MbrCoords[2])
            y1 = float(MbrCoords[3])
            x2 = float(MbrCoords[4])
            y2 = float(MbrCoords[5])
            x3 = float(MbrCoords[6])
            y3 = float(MbrCoords[7])
            Dx0 = x0 - x1
            Dy0 = y0 - y1
            Dx1 = x1 - x2
            Dy1 = y1 - y2
            w2 = ((Dx0 * Dx0) + (Dy0 * Dy0)) ** 0.5
            l2 = ((Dx1 * Dx1) + (Dy1 * Dy1)) ** 0.5
            a2 = w2 * l2
            HA = (pGA / a2) * 100
            RatioL2W = l2 / w2
            row.MBR = HA
            row.L2W = RatioL2W

            #Make a new 4-point polygon for the MBR
            p1 = GP.CreateObject("Point")
            p1.x = x0
            p1.y = y0
            p2 = GP.CreateObject("Point")
            p2.x = x1
            p2.y = y1
            p3 = GP.CreateObject("Point")
            p3.x = x2
            p3.y = y2
            p4 = GP.CreateObject("Point")
            p4.x = x3
            p4.y = y3            
            ##MBR = GP.CreateObject("Polygon")   
            MBR = GP.CreateObject("Array")
            MBR.add(p1)
            MBR.add(p2)
            MBR.add(p3)
            MBR.add(p4)
            row.shape = MBR
            rows.UpdateRow(row)  
            row = rows.Next()

        rows = None
        row = None
        OutputPolys = None
    
    except Exception, ErrorDesc:
        #If an error set output boolean parameter "Error" to True.
        GP.AddError(str(ErrorDesc))
        rows = None
        row = None
        OutputPolys = None